library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;


architecture sample_arch of xillycapture is
  component async_fifo_32
    port (
      rst: IN std_logic;
      wr_clk: IN std_logic;
      rd_clk: IN std_logic;
      din: IN std_logic_VECTOR(31 downto 0);
      wr_en: IN std_logic;
      rd_en: IN std_logic;
      dout: OUT std_logic_VECTOR(31 downto 0);
      full: OUT std_logic;
      empty: OUT std_logic);
  end component;

-- Synplicity black box declaration
  attribute syn_black_box : boolean;
  attribute syn_black_box of async_fifo_32: component is true;

  signal bus_clk :  std_logic;
  signal reset_32 : std_logic;

  signal user_r_read_32_rden :  std_logic;
  signal user_r_read_32_empty :  std_logic;
  signal user_r_read_32_data :  std_logic_vector(31 DOWNTO 0);
  signal user_r_read_32_eof :  std_logic;
  signal user_r_read_32_open :  std_logic;

  --  Wires and registers related to data capturing
  signal capture_clk          : std_logic;
  signal capture_data         : std_logic_vector (31 downto 0);
  signal capture_en           : std_logic;
  signal capture_full         : std_logic;
  signal capture_has_been_full : std_logic;
  signal capture_has_been_nonfull : std_logic;
  signal capture_open         : std_logic;
  signal capture_open_cross   : std_logic;
  signal has_been_full        : std_logic;
  signal has_been_full_cross  : std_logic;
  signal slowdown             : std_logic_vector (4 downto 0);
  signal slowdown_is_zero     : std_logic;
  
begin
  process (capture_clk)
  begin
    if (capture_clk'event and capture_clk = '1') then

--  Bogus data source
      
      if  ( capture_en = '1' ) then
        capture_data <= capture_data + 1 ;
      end if;

--  The slowdown register limits the data pace to 1/32 the bus_clk
--  when capture_clk = bus_clk. This may be necessary, because the
--  core in the evaluation kit is configured for simplicity, and
--  not for performance.
--  The slowdown register has no function in a real-life application.

      slowdown <= slowdown + 1 ;

--  capture_has_been_full remembers that the FIFO has been full
--  until the file is closed. capture_has_been_nonfull prevents
--  capture_has_been_full to respond to the initial full condition
--  every FIFO displays on reset.

      if  ( capture_full = '0' ) then
        capture_has_been_nonfull <= '1' ;
      elsif  ( capture_open = '0' ) then
        capture_has_been_nonfull <= '0' ;
      end if;

      if  ( capture_full = '1' and capture_has_been_nonfull = '1' ) then
        capture_has_been_full <= '1' ;
      elsif  ( capture_open = '0' ) then
        capture_has_been_full <= '0' ;
      end if;

    end if;
  end process;

--  The dependency on slowdown is only for bogus data

  capture_en <= '0' WHEN not (slowdown = 0) ELSE
                capture_open and not capture_full
                and not capture_has_been_full ;

--  Clock crossing logic: bus_clk -> capture_clk

  process (capture_clk)
  begin
    if (capture_clk'event and capture_clk = '1') then
      capture_open_cross <= user_r_read_32_open ;
      capture_open <= capture_open_cross ;
    end if;
  end process;

--  Clock crossing logic: capture_clk -> bus_clk

  process (bus_clk)
  begin
    if (bus_clk'event and bus_clk = '1') then
      has_been_full_cross <= capture_has_been_full ;
      has_been_full <= has_been_full_cross ;
    end if;
  end process;

--  The user_r_read_32_eof signal is required to go from '0' to '1' only on
--  a clock cycle following an asserted read enable, according to Xillybus'
--  core API. This is assured, since it's a logical AND between
--  user_r_read_32_empty and has_been_full. has_been_full goes high when the
--  FIFO is full, so it's guaranteed that user_r_read_32_empty is low when
--  that happens. On the other hand, user_r_read_32_empty is a FIFO's empty
--  signal, which naturally meets the requirement.

  user_r_read_32_eof <= user_r_read_32_empty and has_been_full ;
  
  fifo_32 : async_fifo_32
    port map(
      rst        => reset_32,
      wr_clk     => capture_clk,
      rd_clk     => bus_clk,
      din        => capture_data,
      wr_en      => capture_en,
      rd_en      => user_r_read_32_rden,
      dout       => user_r_read_32_data,
      full       => capture_full,
      empty      => user_r_read_32_empty
      );

  reset_32 <= not user_r_read_32_open;

end sample_arch;
